import { isDefined } from '@collectable/core';
import { RedBlackTreeIterator, PathNode, findByIndex, findIndex, findPathToIndex } from '../internals';
function normalizeIndex(index, size) {
    return index < 0 ? size + index < 0 ? -1 : size + index
        : index >= size ? -1 : index;
}
export function at(index, tree) {
    return (index = normalizeIndex(index, tree._size)) === -1 ? void 0 : findByIndex(index, tree);
}
export function keyAt(index, tree) {
    const node = (index = normalizeIndex(index, tree._size)) === -1 ? void 0 : findByIndex(index, tree);
    return isDefined(node) ? node.key : void 0;
}
export function valueAt(index, tree) {
    const node = (index = normalizeIndex(index, tree._size)) === -1 ? void 0 : findByIndex(index, tree);
    return isDefined(node) ? node.value : void 0;
}
export function indexOf(key, tree) {
    return findIndex(key, tree._root, tree._compare);
}
export function iterateFromIndex(reverse, index, tree) {
    var path = (index = normalizeIndex(index, tree._size)) === -1 ? PathNode.NONE : findPathToIndex(index, tree._root);
    return new RedBlackTreeIterator(path, reverse);
}

//# sourceMappingURL=at.js.map
