import { curry3 } from '@typed/curry';
import { MappableIterator, preventCircularRefs, unwrapAny, isUndefined } from '@collectable/core';
import { isNone } from '../internals';
import { iterateFromFirst, size } from './index';
const NOMAP = () => { };
export function arrayFrom(arg, tree) {
    var map;
    if (isUndefined(tree)) {
        tree = arg;
        map = NOMAP;
    }
    else {
        map = arg;
    }
    var array = new Array(size(tree));
    var it = iterateFromFirst(tree);
    var i, entry;
    if (map === NOMAP) {
        for (i = 0; i < array.length; i++) {
            array[i] = it.next().value;
        }
    }
    else {
        for (i = 0; i < array.length; i++) {
            var entry = it.next().value;
            array[i] = map(entry.value, entry.key, i);
        }
    }
    return array;
}
export function values(tree) {
    var it = iterateFromFirst(tree);
    return new MappableIterator(it, valueOf);
}
export function keys(tree) {
    var it = iterateFromFirst(tree);
    return new MappableIterator(it, keyOf);
}
const newObject = () => ({});
const unwrapShallow = curry3(unwrapTree)(false);
const unwrapDeep = curry3(preventCircularRefs)(newObject, curry3(unwrapTree)(true));
/**
 * Returns the tree unwrapped as a plain JavaScript object. Keys are treated as (or converted to) strings.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {boolean} deep If true, any valid Collectable.js collection values nested in the tree will also be unwrapped
 * @param {RedBlackTree<K, V>} tree The tree to read values from
 * @returns {Associative<V>} A plain JavaScript object containing entries from the tree.
 */
export function unwrap(deep, tree) {
    return deep ? unwrapDeep(tree) : unwrapShallow(tree, newObject());
}
function unwrapTree(deep, tree, target) {
    if (tree._size === 0) {
        return {};
    }
    var stack = new Array(Math.ceil(Math.log(tree._size)) * 2);
    var i = 0, node = tree._root, branch = 1 /* LEFT */;
    do {
        target[node.key] = deep ? unwrapAny(node.value) : node.value;
        if (branch === 1 /* LEFT */) {
            if (!isNone(node._left)) {
                stack[i++] = [node, 2 /* RIGHT */];
                node = node._left;
                branch = 1 /* LEFT */;
                continue;
            }
            else {
                branch = 2 /* RIGHT */;
            }
        }
        if (branch === 2 /* RIGHT */) {
            if (!isNone(node._right)) {
                stack[i++] = [node, 0 /* NONE */];
                node = node._right;
                branch = 1 /* LEFT */;
                continue;
            }
            else {
                branch = 0 /* NONE */;
            }
        }
        if (branch === 0 /* NONE */ && i > 0) {
            [node, branch] = stack[--i];
        }
    } while (i > 0 || branch !== 0 /* NONE */);
    return target;
}
function keyOf(node) {
    return node.key;
}
function valueOf(node) {
    return node.value;
}

//# sourceMappingURL=unwrap.js.map
